@@ -22,13 +22,12 @@ import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 
-import javax.servlet.ServletException;
-import javax.servlet.http.Cookie;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.http.Cookie;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
-import org.apache.catalina.Host;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.Manager;
 import org.apache.catalina.Realm;
@@ -272,14 +271,17 @@ public class SingleSignOn extends ValveBase {
             if (domain != null) {
                 cookie.setDomain(domain);
             }
-            // This is going to trigger a Set-Cookie header. While the value is
-            // not security sensitive, ensure that expectations for secure and
-            // httpOnly are met
+            /*
+             * This is going to trigger a Set-Cookie header. While the value is not security sensitive, ensure that
+             * expectations for secure, httpOnly and Partitioned are met.
+             */
             cookie.setSecure(request.isSecure());
             if (request.getServletContext().getSessionCookieConfig().isHttpOnly() ||
                     request.getContext().getUseHttpOnly()) {
                 cookie.setHttpOnly(true);
             }
+            cookie.setAttribute(Constants.COOKIE_PARTITIONED_ATTR,
+                    Boolean.toString(request.getContext().getUsePartitioned()));
 
             response.addCookie(cookie);
         }
@@ -567,39 +569,12 @@ public class SingleSignOn extends ValveBase {
 
     @Override
     protected void startInternal() throws LifecycleException {
-        Container container = getContainer();
-        while (container != null && !(container instanceof Engine)) {
-            container = container.getParent();
+        Container c = getContainer();
+        while (c != null && !(c instanceof Engine)) {
+            c = c.getParent();
         }
-        if (container != null) {
-            engine = (Engine) container;
-        }
-        // Starting with the associated container, verify it has a realm associated,
-        // and that no child container returns a different realm
-        container = getContainer();
-        Realm containerRealm = container.getRealm();
-        if (containerRealm == null) {
-            containerLog.warn(sm.getString("singleSignOn.noRealm", container.getName()));
-        } else {
-            if (container instanceof Engine) {
-                for (Container host : engine.findChildren()) {
-                    if (host.getRealm() != containerRealm) {
-                        containerLog.warn(sm.getString("singleSignOn.duplicateRealm", host.getName()));
-                    } else {
-                        for (Container context : host.findChildren()) {
-                            if (context.getRealm() != containerRealm) {
-                                containerLog.warn(sm.getString("singleSignOn.duplicateRealm", context.getName()));
-                            }
-                        }
-                    }
-                }
-            } else if (container instanceof Host) {
-                for (Container context : container.findChildren()) {
-                    if (context.getRealm() != containerRealm) {
-                        containerLog.warn(sm.getString("singleSignOn.duplicateRealm", context.getName()));
-                    }
-                }
-            }
+        if (c != null) {
+            engine = (Engine) c;
         }
         super.startInternal();
     }
@@ -610,27 +585,4 @@ public class SingleSignOn extends ValveBase {
         super.stopInternal();
         engine = null;
     }
-
-    protected void sessionChangedId(String ssoId, Session session, String oldSessionId) {
-        if (containerLog.isDebugEnabled()) {
-            containerLog.debug(sm.getString("singleSignOn.debug.sessionChangedId", session, oldSessionId, ssoId));
-        }
-
-        SingleSignOnEntry entry = cache.get(ssoId);
-        if (entry == null) {
-            return;
-        }
-
-        /*
-         * Associate the new sessionId with this SingleSignOnEntry. A SessionListener will be registered for the new
-         * sessionID. If not, then we would not notice any subsequent Session.SESSION_DESTROYED_EVENT for the session.
-         */
-        entry.addSession(this, ssoId, session);
-
-        /*
-         * Remove the obsolete sessionId from the SingleSignOnEntry. The sessionId part of the SingleSignOnSessionKey is
-         * final.
-         */
-        entry.removeSession(session, oldSessionId);
-    }
 }
